@@ -14,10 +14,15 @@ import textops
 import naghelp
 import time
 import subprocess
+import traceback
+import fcntl
+import errno
+import os
+from .tools import Timeout, TimeoutError
 
-__all__ = ['search_invalid_port', 'runsh', 'mrunsh', 'Expect', 'Telnet', 'Ssh', 'Snmp',
-           'Timeout', 'TimeoutError', 'CollectError', 'ConnectionError', 'NotConnected',
-           'UnexpectedResultError']
+__all__ = ['search_invalid_port', 'is_ping_ok', 'runsh', 'runshex', 'mrunsh', 'mrunshex',
+           'Expect', 'Telnet', 'Ssh', 'Sftp', 'Snmp', 'Http',
+           'CollectError', 'ConnectionError', 'NotConnected', 'UnexpectedResultError']
 
 class CollectError(Exception):
     """Exception raised when a collect is unsuccessful
@@ -43,14 +48,6 @@ class ConnectionError(CollectError):
     """
     pass
 
-class TimeoutError(CollectError):
-    """Exception raised when a connection or a collect it too long to process
-
-    It may come from unreachable remote host, too long lasting commands, bad pattern matching on
-    Expect/Telnet/Ssh for connection or prompt search steps.
-    """
-    pass
-
 class UnexpectedResultError(CollectError):
     """Exception raised when a command gave an unexpected result
 
@@ -59,37 +56,12 @@ class UnexpectedResultError(CollectError):
     """
     pass
 
-class Timeout:
-    """Set an execution timeout for a block of code
-
-    It uses process signals, it should not work on windows platforms.
-
-    Args:
-
-        seconds (int): The time in seconds after which a TimeoutError will be raise if the block
-            has not finished its execution.
-        error_message(str): The string to pass to the TimeoutError exception.
-
-    Raises:
-
-        TimeoutError: When the block execution has not finished on-time.
-
-    Examples:
-
-        >>> with timeout(seconds=3):
-        >>>     time.sleep(4)
+class InvalidCommandError(CollectError):
+    """Exception raised when a command to be run is invalid
 
+    Usually, this is raised for empty command
     """
-    def __init__(self, seconds=1, error_message='Timeout'):
-        self.seconds = seconds
-        self.error_message = error_message
-    def handle_timeout(self, signum, frame):
-        raise TimeoutError(self.error_message)
-    def __enter__(self):
-        signal.signal(signal.SIGALRM, self.handle_timeout)
-        signal.alarm(self.seconds)
-    def __exit__(self, type, value, traceback):
-        signal.alarm(0)
+    pass
 
 def search_invalid_port(ip,ports):
     """Returns the first invalid port encountered or None if all are reachable
@@ -122,6 +94,24 @@ def search_invalid_port(ip,ports):
             return port
     return None
 
+def is_ping_ok(ip,timeout=10):
+    """Returns True if the ip pings OK
+
+    Args:
+
+        ip (str): ip address to test
+        timeout (int): timeout after witch the ping is considerated as bad.
+
+    Returns:
+
+        True if ping is OK.
+    """
+    try:
+        out,err,rc = runshex('ping -c 1 %s' % ip,timeout=timeout)
+        return rc == 0
+    except TimeoutError:
+        return False
+
 def _raise_unexpected_result(result, key, cmd, help_str=''):
     if isinstance(result,textops.ListExt):
         result = result.tostr()
@@ -145,7 +135,7 @@ def _filter_result(result, key, cmd, expected_pattern=r'\S', unexpected_pattern=
     if unexpected_pattern:
         if isinstance(unexpected_pattern,basestring):
             unexpected_pattern = re.compile(unexpected_pattern)
-        if result | textops.haspattern(unexpected_pattern):
+        if result and result | textops.haspattern(unexpected_pattern):
             help_str = '-> found the pattern "%s" :\n\n' % unexpected_pattern.pattern
             help_str += result | textops.findhighlight(unexpected_pattern,line_nbr=True,nlines=5).tostr()
             _raise_unexpected_result(result, key, cmd, help_str)
@@ -159,7 +149,23 @@ def _filter_result(result, key, cmd, expected_pattern=r'\S', unexpected_pattern=
             else:
                 _raise_unexpected_result(result, key, cmd, '-> cannot find the pattern "%s"' % expected_pattern.pattern)
 
-    return result
+    return textops.extend_type(result)
+
+def _debug_caller_info():
+    if naghelp.logger.getEffectiveLevel() == naghelp.logging.DEBUG:
+        prev_call = ''
+        stack = traceback.extract_stack()
+        for file,line,func_name,func_line in reversed(stack):
+            file = os.path.basename(file)
+            if file != 'collect.py':
+                return '[%s:%s]' % (file,line),prev_call
+            prev_call = func_name
+    return '',''
+
+def _debug_caller(self):
+    file_line,prev_call = self._debug_caller_info()
+    return file_line
+
 
 def runsh(cmd, context = {}, timeout = 30, expected_pattern=r'\S', unexpected_pattern=None, filter=None, key='' ):
     r"""Run a local command with a timeout
@@ -218,9 +224,74 @@ def runsh(cmd, context = {}, timeout = 30, expected_pattern=r'\S', unexpected_pa
         >>> print l
         ['ls: cannot access /etc/does_not_exist: No such file or directory']
     """
+    stdout, stderr, rc = runshex(cmd, context = context, timeout = timeout,
+                               expected_pattern = expected_pattern,
+                               unexpected_pattern = unexpected_pattern, filter=filter, key=key,
+                               unexpected_stderr = False )
+    return stdout.splitlines()
+
+def runshex(cmd, context = {}, timeout = 30, expected_pattern=r'\S', unexpected_pattern=None,filter=None, key='',unexpected_stderr=True ):
+    r"""Run a local command with a timeout
+
+    | If the command is a string, it will be executed within a shell.
+    | If the command is a list (the command and its arguments), the command is executed without a shell.
+    | If a context dict is specified, the command is formatted with that context (:meth:`str.format`)
+
+    Args:
+
+        cmd (str or a list): The command to run
+        context (dict): The context to format the command to run (Optional)
+        timeout (int): The timeout in seconds after with the forked process is killed
+            and TimeoutException is raised (Default : 30s).
+        expected_pattern (str or regex): raise UnexpectedResultError if the pattern is not found.
+            if None, there is no test. By default, tests the result is not empty.
+        unexpected_pattern (str or regex): raise UnexpectedResultError if the pattern is found
+            if None, there is no test. By default, there is no test.
+        filter (callable): call a filter function with ``result, key, cmd`` parameters.
+            The function should return the modified result (if there is no return statement,
+            the original result is used).
+            The filter function is also the place to do some other checks : ``cmd`` is the command
+            that generated the ``result`` and ``key`` the key in the dictionary for ``mrun``,
+            ``mget`` and ``mwalk``.
+            By Default, there is no filter.
+        key (str): a key string to appear in UnexpectedResultError if any.
+        unexpected_stderr (bool): When True (Default), it raises an error if stderr is not empty
+
+    Returns:
+
+        tuple: stdout, stderr, return code tuple
+
+    Note:
+
+        It returns **ONLY** stdout. If you want to get stderr, you need to redirect it to stdout.
+    """
+    if not cmd:
+        raise InvalidCommandError('Command is empty')
+
     with Timeout(seconds=timeout, error_message='Timeout (%ss) for command : %s' % (timeout,cmd)):
-        result = textops.run(cmd, context).l
-        return _filter_result(result, key, cmd, expected_pattern, unexpected_pattern, filter)
+        if isinstance(cmd, basestring):
+            if context:
+                cmd = cmd.format(**context)
+            p=subprocess.Popen(['timeout','%ss' % timeout,'sh','-c',cmd],stdout=subprocess.PIPE,stderr=subprocess.PIPE)
+        elif isinstance(cmd, list):
+            if context:
+                cmd = [ i.format(**context) for i in cmd ]
+            if cmd[0] != 'timeout':
+                cmd[0:0] = ['timeout','%ss' % timeout]
+            p=subprocess.Popen(cmd,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
+        stdout_msg = ''
+        stderr_msg = ''
+        while p.returncode is None:
+            (stdout, stderr) = p.communicate()
+            stdout_msg += stdout
+            stderr_msg += stderr
+        if isinstance(cmd, unicode):
+            cmd=cmd.encode('utf-8','replace')
+        else:
+            cmd=str(cmd)
+        if unexpected_stderr and stderr_msg:
+            _raise_unexpected_result(stderr_msg, key, cmd, help_str='<stderr> returned :')
+        return _filter_result(stdout_msg, key, cmd, expected_pattern, unexpected_pattern, filter),stderr_msg,p.returncode
 
 def mrunsh(cmds, context = {},cmd_timeout = 30, total_timeout = 60, expected_pattern=r'\S', unexpected_pattern=None, filter=None):
     r"""Run multiple local commands with timeouts
@@ -274,6 +345,60 @@ def mrunsh(cmds, context = {},cmd_timeout = 30, total_timeout = 60, expected_pat
             dct[k] = runsh(cmd, context, cmd_timeout, expected_pattern, unexpected_pattern, filter, k)
         return dct
 
+def mrunshex(cmds, context = {},cmd_timeout = 30, total_timeout = 60, expected_pattern=r'\S', unexpected_pattern=None, filter=None,unexpected_stderr=True):
+    r"""Run multiple local commands with timeouts
+
+    It works like :func:`runshex` except that one must provide a dictionary of commands.
+    It will generate the same dictionary where values will be replaced by command execution output.
+    It is possible to specify a by-command timeout and a global timeout for the whole dictionary.
+    stderr are store in ``<key>_stderr`` and return codes in ``<key>_rcode``
+
+    Args:
+
+        cmds (dict): dictionary where values are the commands to execute.
+
+            | If the command is a string, it will be executed within a shell.
+            | If the command is a list (the command and its arguments), the command is executed without a shell.
+            | If a context dict is specified, the command is formatted with that context (:meth:`str.format`)
+
+        context (dict): The context to format the command to run
+        cmd_timeout (int): The timeout in seconds for a single command
+        total_timeout (int): The timeout in seconds for the all commands
+        expected_pattern (str or regex): raise UnexpectedResultError if the pattern is not found.
+            if None, there is no test. By default, tests the result is not empty.
+        unexpected_pattern (str or regex): raise UnexpectedResultError if the pattern is found
+            if None, there is no test. By default, there is no test.
+        filter (callable): call a filter function with ``result, key, cmd`` parameters.
+            The function should return the modified result (if there is no return statement,
+            the original result is used).
+            The filter function is also the place to do some other checks : ``cmd`` is the command
+            that generated the ``result`` and ``key`` the key in the dictionary for ``mrun``,
+            ``mget`` and ``mwalk``.
+            By Default, there is no filter.
+        unexpected_stderr (bool): When True (Default), it raises an error if stderr is not empty
+
+    Returns:
+
+        :class:`textops.DictExt`: Dictionary where each value is the Command execution stdout as a list of lines.
+
+    Note:
+
+        Command execution returns **ONLY** stdout. If you want to get stderr, you need to redirect it to stdout.
+
+    Examples:
+
+        >>> mrunsh({'now':'LANG=C date','quisuisje':'whoami'})
+        {'now': ['Wed Dec 16 11:50:08 CET 2015'], 'quisuisje': ['elapouya']}
+
+    """
+    with Timeout(seconds=total_timeout, error_message='Timeout (%ss) for mrunsh commands : %s' % (total_timeout,cmds)):
+        dct = textops.DictExt()
+        if isinstance(cmds,dict):
+            cmds = cmds.items()
+        for k,cmd in cmds:
+            dct[k],dct[k+'_stderr'],dct[k+'_rcode'] = runshex(cmd, context, cmd_timeout, expected_pattern, unexpected_pattern, filter, k,unexpected_stderr)
+        return dct
+
 def debug_pattern_list(pat_list):
     return [ (pat if isinstance(pat,basestring) else pat.pattern) for pat in pat_list ]
 
@@ -415,11 +540,11 @@ class Expect(object):
         import pexpect
         self.in_with = False
         self.is_connected = False
-        naghelp.logger.debug('#### Expect( %s ) ###############',spawn)
+        naghelp.logger.debug('collect -> #### Expect( %s ) ###############',spawn)
         with Timeout(seconds = timeout, error_message='Timeout (%ss) for pexpect : %s' % (timeout,spawn)):
             self.child = pexpect.spawn(spawn)
             if login_steps or prompt:
-                naghelp.logger.debug('==== Login steps up to the prompt =====')
+                naghelp.logger.debug('collect -> ==== Login steps up to the prompt =====')
                 error_msg = self._expect_steps( (login_steps or ()) + ( ((prompt,None),) if prompt else () ) )
                 if error_msg:
                     raise ConnectionError(error_msg)
@@ -436,7 +561,7 @@ class Expect(object):
         nb_steps = len(steps)
         infinite_loop_detect = 0
         while step < nb_steps:
-            naghelp.logger.debug('--------- STEP #%s--------------------------',step)
+            naghelp.logger.debug('collect -> --------- STEP #%s--------------------------',step)
             expects = steps[step]
             # normalizing : transform tuple of string into tuple of tuples of string
             if isinstance(expects[0],basestring):
@@ -452,22 +577,22 @@ class Expect(object):
                         next_expects = (next_expects,)
                     expects += next_expects
                 patterns = [ self._expect_pattern_rewrite(e[0]) for e in expects ]
-                naghelp.logger.debug('<-- expect(%s) ...',patterns)
+                naghelp.logger.debug('collect -> <-- expect(%s) ...',patterns)
                 try:
                     found = self.child.expect(patterns)
                 except pexpect.EOF:
                     naghelp.logger.debug('CollectError : No more data (EOF) from %s' % self.spawn)
                     raise CollectError('No more data (EOF) from %s' % self.spawn)
-                naghelp.logger.debug('  --> found : "%s"',patterns[found])
+                naghelp.logger.debug('collect ->   --> found : "%s"',patterns[found])
             to_send = expects[found][1]
             if to_send is not None:
                 if isinstance(to_send,basestring):
                     to_send = to_send.format(**self.context)
                     if to_send and to_send[-1] == '\n':
-                        naghelp.logger.debug('  ==> sendline : %s',to_send[:-1])
+                        naghelp.logger.debug('collect ->   ==> sendline : %s',to_send[:-1])
                         self.child.sendline(to_send[:-1])
                     else:
-                        naghelp.logger.debug('  ==> send : %s',to_send)
+                        naghelp.logger.debug('collect ->   ==> send : %s',to_send)
                         self.child.send(to_send)
                 elif to_send == Expect.KILL:
                     return_msg = self.child.before+'.'
@@ -488,7 +613,7 @@ class Expect(object):
                 naghelp.logger.debug('Too many expect for %s',patterns)
                 raise CollectError('Too many expect for %s' % patterns)
 
-        naghelp.logger.debug('FINISHED steps')
+        naghelp.logger.debug('collect -> FINISHED steps')
         return ''
 
     def __enter__(self):
@@ -510,15 +635,15 @@ class Expect(object):
                 self.child.kill(0)
             except OSError:
                 pass
-            naghelp.logger.debug('#### Expect : Connection closed ###############')
+            naghelp.logger.debug('collect -> #### Expect : Connection closed ###############')
 
     def _run_cmd(self,cmd):
         if cmd:
-            naghelp.logger.debug('  ==> sendline : %s',cmd)
+            naghelp.logger.debug('collect -> run("%s") %s',cmd,naghelp.debug_caller())
             self.child.sendline('%s' % cmd)
 
         prompt = self._expect_pattern_rewrite(self.prompt)
-        naghelp.logger.debug('<-- expect prompt : %s',prompt)
+        naghelp.logger.debug('collect ->     expect prompt : %s',prompt)
         try:
             self.child.expect(prompt)
         except pexpect.EOF:
@@ -738,35 +863,35 @@ class Telnet(object):
             raise ConnectionError('No host specified for Telnet')
         if not user:
             raise ConnectionError('No user specified for Telnet')
-        naghelp.logger.debug('#### Telnet( %s@%s ) ###############',user, host)
+        naghelp.logger.debug('collect -> #### Telnet( %s@%s ) ###############',user, host)
         with Timeout(seconds = timeout, error_message='Timeout (%ss) for telnet to %s' % (timeout,host)):
             try:
                 self.tn = telnetlib.Telnet(host,port,timeout,**kwargs)
                 #self.tn.set_debuglevel(1)
             except Exception,e:
                 raise ConnectionError(e)
-            naghelp.logger.debug('<-- expect(%s) ...',debug_pattern_list(login_pattern))
+            naghelp.logger.debug('collect -> <-- expect(%s) ...',debug_pattern_list(login_pattern))
             time.sleep(sleep_login or sleep)
             self.tn.expect(login_pattern)
-            naghelp.logger.debug('  ==> %s',user)
+            naghelp.logger.debug('collect ->   ==> %s',user)
             time.sleep(sleep)
             self.tn.write(user + "\n")
-            naghelp.logger.debug('<-- expect(%s) ...',debug_pattern_list(passwd_pattern))
+            naghelp.logger.debug('collect -> <-- expect(%s) ...',debug_pattern_list(passwd_pattern))
             if password is not None:
                 time.sleep(sleep)
                 self.tn.expect(passwd_pattern)
-                naghelp.logger.debug('  ==> (hidden password)')
+                naghelp.logger.debug('collect ->   ==> (hidden password)')
                 time.sleep(sleep)
                 self.tn.write(password + "\n")
-            naghelp.logger.debug('<-- expect(%s) ...',debug_pattern_list(prompt_pattern + autherr_pattern))
+            naghelp.logger.debug('collect -> <-- expect(%s) ...',debug_pattern_list(prompt_pattern + autherr_pattern))
             time.sleep(sleep)
             pat_id,m,buffer = self.tn.expect(prompt_pattern + autherr_pattern)
-            naghelp.logger.debug('pat_id,m,buffer = %s, %s, %s',pat_id,m,buffer)
+            naghelp.logger.debug('collect -> pat_id,m,buffer = %s, %s, %s',pat_id,m,buffer)
             if pat_id < 0:
                 raise ConnectionError('No regular prompt found.')
             if pat_id >= len(prompt_pattern):
                 raise ConnectionError('Authentication error')
-            naghelp.logger.debug('Prompt found : is_connected = True')
+            naghelp.logger.debug('collect -> Prompt found : is_connected = True')
             self.is_connected = True
 
     @staticmethod
@@ -791,15 +916,15 @@ class Telnet(object):
         if not self.in_with:
             self.tn.close()
             self.is_connected = False
-            naghelp.logger.debug('#### Telnet : Connection closed ###############')
+            naghelp.logger.debug('collect -> #### Telnet : Connection closed ###############')
 
     def _run_cmd(self,cmd):
         if isinstance(cmd, unicode):
             cmd = cmd.encode('utf-8','ignore')
-        naghelp.logger.debug('  ==> %s',cmd)
+        naghelp.logger.debug('collect -> run("%s") %s',cmd,naghelp.debug_caller())
         time.sleep(self.sleep)
         self.tn.write('%s\n' % cmd)
-        naghelp.logger.debug('<-- expect(%s) ...',debug_pattern_list(self.prompt_pattern))
+        naghelp.logger.debug('collect -> <-- expect(%s) ...',debug_pattern_list(self.prompt_pattern))
         time.sleep(self.sleep)
         pat_id,m,buffer = self.tn.expect(self.prompt_pattern)
         out = buffer.replace('\r','')
@@ -809,7 +934,7 @@ class Telnet(object):
         # remove cmd and prompt (first and last line)
         out = out.splitlines()[:-1]
         cmd_out = '\n'.join(out)
-        naghelp.logger.debug('\n%s\n%s\n%s','v'*80,cmd_out,'^'*80)
+        naghelp.debug_listing(cmd_out)
         return cmd_out
 
     def run(self, cmd, timeout=30, auto_close=True, expected_pattern=0, unexpected_pattern=0, filter=0, **kwargs):
@@ -1007,7 +1132,7 @@ class Ssh(object):
         if auto_accept_new_host:
             self.client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
         self.client.load_system_host_keys()
-        naghelp.logger.debug('#### Ssh( %s@%s ) ###############',user, host)
+        naghelp.logger.debug('collect -> #### Ssh( %s@%s ) ###############',user, host)
         try:
             self.client.connect(host,username=user,password=password, timeout=timeout, **kwargs)
             if self.prompt_pattern:
@@ -1017,7 +1142,7 @@ class Ssh(object):
                 self._read_to_prompt()
         except Exception,e:
             raise ConnectionError(e)
-        naghelp.logger.debug('is_connected = True')
+        naghelp.logger.debug('collect -> is_connected = True')
         self.is_connected = True
 
     def __enter__(self):
@@ -1032,7 +1157,7 @@ class Ssh(object):
         if not self.in_with:
             self.client.close()
             self.is_connected = False
-            naghelp.logger.debug('#### Ssh : Connection closed ###############')
+            naghelp.logger.debug('collect -> #### Ssh : Connection closed ###############')
 
     def _read_to_prompt(self):
         buff = ''
@@ -1041,13 +1166,13 @@ class Ssh(object):
         return buff
 
     def _run_cmd(self,cmd,timeout):
-        naghelp.logger.debug('  ==> %s',cmd)
+        naghelp.logger.debug('collect -> run("%s") %s',cmd,naghelp.debug_caller())
         if self.prompt_pattern is None:
             stdin, stdout, stderr = self.client.exec_command(cmd,timeout=timeout,get_pty=self.get_pty)
             out = stdout.read()
             if self.add_stderr:
                 out += stderr.read()
-            naghelp.logger.debug('\n%s\n%s\n%s','v'*80,out,'^'*80)
+            naghelp.debug_listing(out)
             return out
         else:
             self.chan.send('%s\n' % cmd)
@@ -1059,7 +1184,7 @@ class Ssh(object):
             # remove cmd and prompt (first and last line)
             out = out.splitlines()[:-1]
             cmd_out = '\n'.join(out)
-            naghelp.logger.debug('\n%s\n%s\n%s','v'*80,cmd_out,'^'*80)
+            naghelp.debug_listing(cmd_out)
             return cmd_out
 
     def run(self, cmd, timeout=30, auto_close=True, expected_pattern=0, unexpected_pattern=0, filter=0, **kwargs):
@@ -1113,9 +1238,9 @@ class Ssh(object):
             out = '<timeout>'
         if auto_close:
             self.close()
-        return textops.StrExt(_filter_result(out,'',cmd, expected_pattern if expected_pattern != 0 else self.expected_pattern,
+        return _filter_result(out,'',cmd, expected_pattern if expected_pattern != 0 else self.expected_pattern,
                                                          unexpected_pattern if unexpected_pattern != 0 else self.unexpected_pattern,
-                                                         filter if filter != 0 else self.filter))
+                                                         filter if filter != 0 else self.filter)
 
     def mrun(self, cmds, timeout=30, auto_close=True, expected_pattern=0, unexpected_pattern=0, filter=0, **kwargs):
         r"""Execute many commands at the same time
@@ -1187,6 +1312,109 @@ class Ssh(object):
             self.close()
         return dct
 
+class Sftp(object):
+    r"""Sftp class helper
+
+    This class is a wrapper around the paramiko sftp client, see
+    `sftp client documentation <http://docs.paramiko.org/en/2.0/api/sftp.html>`_ for available methods.
+
+    Args:
+
+        host (str): IP address or hostname to connect to
+        user (str): The username to use for login
+        password (str): The password
+        timeout (int): Time in seconds before raising an error or a None value
+        prompt_pattern (str): None by Default. If defined, the way to run commands is to capture
+            the command output up to the prompt pattern. If not defined, it uses paramiko exec_command()
+            method (preferred way).
+        get_pty (bool): Create a pty, this is useful for some ssh connection (Default: False)
+        expected_pattern (str or regex): raise UnexpectedResultError if the pattern is not found
+            in methods that collect data (like run,mrun,get,mget,walk,mwalk...)
+            if None, there is no test. By default, tests the result is not empty.
+        unexpected_pattern (str or regex): raise UnexpectedResultError if the pattern is found
+            if None, there is no test. By default, it tests <timeout>.
+        filter (callable): call a filter function with ``result, key, cmd`` parameters.
+            The function should return the modified result (if there is no return statement,
+            the original result is used).
+            The filter function is also the place to do some other checks : ``cmd`` is the command
+            that generated the ``result`` and ``key`` the key in the dictionary for ``mrun``,
+            ``mget`` and ``mwalk``.
+            By Default, there is no filter.
+        add_stderr (bool): If True, the stderr will be added at the end of results (Default: True)
+        port (int): port number to use (Default : 0 = 22)
+        pkey (PKey): an optional private key to use for authentication
+        key_filename (str):
+            the filename, or list of filenames, of optional private key(s) to
+            try for authentication
+        allow_agent (bool): set to False to disable connecting to the SSH agent
+        look_for_keys (bool): set to False to disable searching for discoverable private key
+            files in ``~/.ssh/``
+        compress (bool): set to True to turn on compression
+        sock (socket): an open socket or socket-like object (such as a `.Channel`) to use
+            for communication to the target host
+        gss_auth (bool): ``True`` if you want to use GSS-API authentication
+        gss_kex (bool):  Perform GSS-API Key Exchange and user authentication
+        gss_deleg_creds (bool): Delegate GSS-API client credentials or not
+        gss_host (str): The targets name in the kerberos database. default: hostname
+        banner_timeout (float): an optional timeout (in seconds) to wait
+            for the SSH banner to be presented.
+
+        Example:
+
+            s=Sftp('localhost','mylogin','mypasswd')
+            s.chdir('remotedir')
+            os.chdir('localdir')
+            s.get('remotefile','mylocalfile')
+            s.close()
+    """
+    def __init__(self,host, user, password=None, timeout=30, auto_accept_new_host=True,
+                 prompt_pattern=None, get_pty=False, *args,**kwargs):
+        #import is done only on demand, because it takes some little time
+        import paramiko
+        self.in_with = False
+        self.is_connected = False
+        self.prompt_pattern = prompt_pattern
+        self.get_pty = get_pty
+        self.client = paramiko.SSHClient()
+        if not host:
+            raise ConnectionError('No host specified for Ssh')
+        if not user:
+            raise ConnectionError('No user specified for Ssh')
+        if auto_accept_new_host:
+            self.client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
+        self.client.load_system_host_keys()
+        naghelp.logger.debug('collect -> #### Sftp( %s@%s ) ###############',user, host)
+        try:
+            self.client.connect(host,username=user,password=password, timeout=timeout, **kwargs)
+            self.sftp = self.client.open_sftp()
+        except Exception,e:
+            raise ConnectionError(e)
+        naghelp.logger.debug('collect -> is_connected = True')
+        self.is_connected = True
+
+    def __enter__(self):
+        self.in_with = True
+        return self
+
+    def __exit__(self, type, value, traceback):
+        self.in_with = False
+        self.close()
+
+    def __getattr__(self, attr):
+        meth = getattr(self.sftp,attr,None)
+        if callable(meth):
+            if not self.is_connected:
+                raise NotConnected('No sftp connection to run your command.')
+            return meth
+        raise AttributeError
+
+    def close(self):
+        if not self.in_with:
+            self.client.close()
+            self.is_connected = False
+            naghelp.logger.debug('collect -> #### Sftp : Connection closed ###############')
+
+
 class Snmp(object):
     r"""Snmp class helper
 
@@ -1330,6 +1558,7 @@ class Snmp(object):
                 >>> snmp.get(('SNMPv2-MIB', 'sysDescr', 0))
                 'SunOS zeus.snmplabs.com 4.1.3_U1 1 sun4m'
         """
+        naghelp.logger.debug('collect -> get(%s) %s',oid_or_mibvar,naghelp.debug_caller())
         oid_or_mibvar = self.normalize_oid(oid_or_mibvar)
         args = list(self.cmd_args)
         args.append(oid_or_mibvar)
@@ -1368,6 +1597,7 @@ class Snmp(object):
                  ...
 
         """
+        naghelp.logger.debug('collect -> walk(%s) %s',oid_or_mibvar,naghelp.debug_caller())
         oid_or_mibvar = self.normalize_oid(oid_or_mibvar)
         lst = textops.ListExt()
         args = list(self.cmd_args)
@@ -1416,6 +1646,57 @@ class Snmp(object):
             dct[var] = self.walk(oid)
         return dct
 
+    def dwalk(self,oid_or_mibvar,irow=-2,icol=-1,cols=None):
+        walk_data = self.walk(oid_or_mibvar)
+        dct={}
+        for oid,val in walk_data:
+            oid_bits = str(oid).split('.')
+            row=int(oid_bits[irow])
+            col=int(oid_bits[icol])
+            dct.setdefault(row,{}).setdefault(col,val)
+        return textops.DictExt(dct)
+
+    def twalk(self,oid_or_mibvar,irow=-2,icol=-1,cols=None):
+        walk_data = self.walk(oid_or_mibvar)
+        dct={}
+        table=textops.ListExt()
+        for oid,val in walk_data:
+            oid_bits = str(oid).split('.')
+            row=int(oid_bits[irow])
+            col=int(oid_bits[icol])
+            dct.setdefault(row,{}).setdefault(col,val)
+        if cols is None:
+            for row_id,rec_dct in sorted(dct.items()):
+                table.append( [ row_id ] + [ rec_dct.get(c,NoAttr) for c in sorted(rec_dct) ] )
+        elif isinstance(cols,(list,tuple)):
+            for row_id,rec_dct in sorted(dct.items()):
+                table.append( [ row_id ] + [ rec_dct.get(c,NoAttr) for c in cols ] )
+        elif isinstance(cols,dict):
+            for row_id,rec_dct in sorted(dct.items()):
+                table.append( dict([ (k,rec_dct.get(v,NoAttr)) for k,v in cols.items() ],_row=row_id) )
+        return table
+
+    def jwalk(self,*twalks_args):
+        dct={}
+        tables = textops.ListExt([ self.twalk(*twalk_args) for twalk_args in twalks_args ])
+        if isinstance(twalks_args[0][-1],(list,tuple,type(None))):
+            for args in twalks_args:
+                assert isinstance(args[-1],(list,tuple,type(None))), 'All wanted columns specifications must be lists/tuples/None'
+            for table in tables:
+                for row in table:
+                    row_id = row[0]
+                    l=dct.setdefault(row_id,[row_id])
+                    l+=row[1:]
+            return textops.ListExt(sorted(dct.values(),key=lambda v:v[0]))
+        else:
+            for args in twalks_args:
+                assert isinstance(args[-1],dict), 'All wanted columns specifications must be dicts'
+            for table in tables:
+                for row in table:
+                    row_id = row['_row']
+                    dct.setdefault(row_id,{}).update(row)
+            return textops.ListExt(sorted(dct.values(),key=lambda v:v['_row']))
+
     def get_oid_range(self,oid_range):
         oids = []
         if oid_range.count('-') == 1:
@@ -1459,6 +1740,7 @@ class Snmp(object):
              'other' : ['value for 1.3.6.1.2.1.1.2.0', 'value for 1.3.6.1.2.1.1.3.0', etc... ] }
 
         """
+        naghelp.logger.debug('collect -> mget(...) %s',naghelp.debug_caller())
         dct = textops.DictExt()
         oid_to_var = {}
         args = list(self.cmd_args)
@@ -1492,3 +1774,105 @@ class Snmp(object):
             else:
                 dct[var] = val
         return dct
+
+    def exists(self,oid_or_mibvar):
+        """Return True if the OID exists
+
+        It return False if the OID does not exists or raise an exception if snmp server is unreachable
+
+        Args:
+
+            oid_or_mibvar (str or ObjectIdentity): an OID path or a pysnmp ObjectIdentity
+
+        Returns:
+
+            bool: True if OID exists
+
+        Examples:
+
+            To collect a numerical OID::
+
+                >>> snmp = Snmp('demo.snmplabs.com')
+                >>> snmp.exists('1.3.6.1.2.1.1.1.0')
+                True
+                >>> snmp.exists('1.3.6.1.2.1.1.1.999')
+                False
+        """
+        oid_or_mibvar = self.normalize_oid(oid_or_mibvar)
+        args = list(self.cmd_args)
+        args.append(oid_or_mibvar)
+        errorIndication, errorStatus, errorIndex, varBinds = self.cmdGenerator.getCmd(*args)
+        if errorIndication or errorStatus:
+            return False
+        return True
+
+class Http(object):
+    r"""Http class helper
+
+    This class helps to collect web pages.
+
+    Args:
+
+        host (str): IP address or hostname to connect to
+        port (int): port number to use (Default : 80 TCP)
+        timeout (int): Time in seconds before raising an error or a None value
+    """
+    def __init__(self, expected_pattern=r'\S', unexpected_pattern=r'<timeout>',
+                 filter=None,*args,**kwargs):
+        import requests
+        self.requests = requests
+        self.expected_pattern = expected_pattern
+        self.unexpected_pattern = unexpected_pattern
+        self.filter = filter
+        self.kwargs = kwargs
+
+    def _get(self,url,*args,**kwargs):
+        naghelp.logger.debug('collect -> get("%s") %s',url,naghelp.debug_caller())
+        params = dict(self.kwargs)
+        params.update(kwargs)
+        try:
+            r = self.requests.get(url,**params)
+        except self.requests.Timeout,e:
+            raise ConnectionError(e)
+        return r.text if r.status_code==200 else ''
+
+    def get(self,url, expected_pattern=0, unexpected_pattern=0, filter=0,*args,**kwargs):
+        """get one URL
+
+        Args:
+
+            url (str): The url to get
+            timeout (int): Time in seconds before raising an error or a None value
+
+        Returns:
+
+            str: The page or NoAttr if URL is reachable but returned a Http Error
+        """
+        out = self._get(url,*args,**kwargs)
+        return _filter_result(out,'','GET %s' % url, expected_pattern if expected_pattern != 0 else self.expected_pattern,
+                                                         unexpected_pattern if unexpected_pattern != 0 else self.unexpected_pattern,
+                                                         filter if filter != 0 else self.filter)
+
+    def mget(self,urls, expected_pattern=0, unexpected_pattern=0, filter=0,*args,**kwargs):
+        """Get multiple URLs at the same time
+
+        Args:
+
+            urls (dict): The urls to get
+            timeout (int): Time in seconds before raising an error or a None value
+
+        Returns:
+
+            :class:`textops.DictExt`: List of pages or NoAttr if not availables
+        """
+        naghelp.logger.debug('collect -> mget(...) %s',naghelp.debug_caller())
+        dct = textops.DictExt()
+        if isinstance(cmds,dict):
+            cmds = cmds.items()
+        for k,cmd in cmds:
+            if k:
+                out = self._get(url,*args,**kwargs)
+                dct[k] = _filter_result(out,k,url, expected_pattern if expected_pattern != 0 else self.expected_pattern,
+                                                                 unexpected_pattern if unexpected_pattern != 0 else self.unexpected_pattern,
+                                                                 filter if filter != 0 else self.filter)
+        return dct
\ No newline at end of file
