@@ -8,8 +8,10 @@
 
 from naghelp import *
 from textops import *
+import re
+import time,datetime
 
-__all__ = ['GaugeMixin','GaugeException']
+__all__ = ['GaugeMixin','GaugeException','HostsManagerMixin']
 
 class GaugeException(Exception):
     pass
@@ -112,8 +114,7 @@ class GaugeMixin(object):
             Test gauge : Temperature=110C
             <BLANKLINE>
         """
-        self.response.add_more('%s : %s',label,value)
-        self.debug('Gauge id=%s, value=%s (warn_min=%s,crit_min=%s,warn_max=%s,crit_max=%s)',id,value,warn_min,crit_min,warn_max,crit_max)
+        self.response.add_more('%s : %s',label,value,no_debug=True)
         if isinstance(value,basestring):
             value = find_pattern.op(value,r'(-?[\d,\.]+)').replace(',','.')
             if value:
@@ -121,6 +122,7 @@ class GaugeMixin(object):
                     value=float(value)
                 else:
                     value=int(value)
+        self.debug('response -> Gauge id=%s, value=%s (warn_min=%s,crit_min=%s,warn_max=%s,crit_max=%s)',id,value,warn_min,crit_min,warn_max,crit_max)
         if isinstance(value,(int,float)):
             if isinstance(crit_min,(int,float)) and value <= crit_min:
                 self.response.add(CRITICAL,'%s : %s <= MIN CRITICAL (%s)' % (label, value, crit_min))
@@ -187,10 +189,10 @@ class GaugeMixin(object):
             Temperature cursor : 21
             >>> p.doctest_end()
         """
-        self.response.add_more('%s : %s',label,value)
+        self.response.add_more('%s : %s',label,value,no_debug=True)
         etalon_name = id + '_etalon'
         etalon_value = self.host.get(etalon_name,None)
-        self.debug('Gauge id=%s, was:%s, now:%s',id,etalon_value,value)
+        self.debug('response -> Gauge id=%s, was:%s, now:%s',id,etalon_value,value)
         if etalon_value is not None and value != etalon_value:
             self.response.add(level,'%s : actual value (%s) has changed (was %s)' % (label, value, etalon_value))
         if value not in [ NoAttr, None ]:
@@ -254,10 +256,10 @@ class GaugeMixin(object):
             Temperature cursor : 19
             >>> p.doctest_end()                          # only for doctest
         """
-        self.response.add_more('%s : %s',label,value)
+        self.response.add_more('%s : %s',label,value,no_debug=True)
         etalon_name = id + '_etalon'
         etalon_value = self.host.get(etalon_name,None)
-        self.debug('Gauge id=%s, was:%s, now:%s',id,etalon_value,value)
+        self.debug('response -> Gauge id=%s, was:%s, now:%s',id,etalon_value,value)
         if etalon_value is not None and value < etalon_value:
             self.response.add(level,'%s : actual value (%s) is less than the reference value (%s)' % (label, value, etalon_value))
         if isinstance(value,(int,float)):
@@ -321,10 +323,10 @@ class GaugeMixin(object):
             Temperature cursor : 21
             >>> p.doctest_end()                          # only for doctest
         """
-        self.response.add_more('%s : %s',label,value)
+        self.response.add_more('%s : %s',label,value,no_debug=True)
         etalon_name = id + '_etalon'
         etalon_value = self.host.get(etalon_name,None)
-        self.debug('Gauge id=%s, was:%s, now:%s',id,etalon_value,value)
+        self.debug('response -> Gauge id=%s, was:%s, now:%s',id,etalon_value,value)
         if etalon_value is not None and value > etalon_value:
             self.response.add(level,'%s : actual value (%s) is more than the reference value (%s)' % (label, value, etalon_value))
         if isinstance(value,(int,float)):
@@ -353,3 +355,118 @@ class GaugeMixin(object):
         """
         etalon_name = id + '_etalon'
         self.host.set(etalon_name,value)
+
+class HostsManagerMixin(object):
+    managed_default_level = OK
+    managed_service_description = 'ManagedHost'
+    managed_response_retention_delta = 0
+    managed_hosts = None
+    managed_data_filename = '/tmp/managed_hosts.json'
+
+    def __init__(self,*args,**kwargs):
+        from pynag.Control import Command
+        from pynag import Model
+        self.pynag_cmd = Command
+        self.pynag_model = Model
+        super(HostsManagerMixin,self).__init__(*args,**kwargs)
+
+    def build_manager_response(self,data):
+        pass
+
+    def build_managed_responses(self,data):
+        pass
+
+    def get_managed_data_filename(self):
+        return self.managed_data_filename
+
+    def clean_managed_host_data(self,hostname):
+        """Method to clean old managed data after loading them"""
+
+    def get_managed_host_data(self,hostname):
+        hostname = self.normalize_hostname(hostname)
+        return self.get_managed_hosts_data().setdefault(hostname,DictExt())
+
+    def get_managed_hosts_data(self):
+        return self.managed_data.setdefault('hosts',DictExt())
+
+    def load_managed_data(self):
+        self.managed_data=self.load_data(self.get_managed_data_filename())
+        self.clean_managed_host_data()
+
+    def normalize_hostname(self,name):
+        if not name:
+            return 'noname'
+        return re.sub(r'[^\w-]+','_',name.strip())
+
+    def is_managed_host(self, hostname_or_serial):
+        if not hostname_or_serial:
+            return False
+        hostname_or_serial = self.normalize_hostname(hostname_or_serial)
+        if not hostname_or_serial:
+            return False
+        if hostname_or_serial in self.managed_data.hosts:
+            return True
+        hostname = self.managed_data.serials[hostname_or_serial]
+        return hostname and hostname in self.managed_data.hosts
+
+    def save_managed_data(self):
+        for hostname, response in self.managed_responses.items():
+            managed_host = self.managed_data.hosts[hostname]
+            managed_host.prev_hash = managed_host.new_hash
+            managed_host.new_hash = response.get_hash()
+            managed_host.prev_state = managed_host.new_state
+            managed_host.new_state = response.get_current_level().exit_code
+            managed_host.updated = int(time.time())
+        self.save_data(self.get_managed_data_filename(),self.managed_data)
+
+    def get_managed_nagios_states(self):
+        return dict([(srv.host_name,int(srv.get_current_status().current_state)) for srv in self.pynag_model.Service.objects.filter(service_description=self.managed_service_description)])
+
+    def init_managed_hosts(self,data):
+        self.managed_responses = {}
+        self.managed_nagios_states = self.get_managed_nagios_states()
+        self.managed_lock = Lockfile(self.get_managed_data_filename())
+        self.managed_lock.acquire()
+        self.load_managed_data()
+
+    def get_managed_response(self,hostname):
+        hostname = self.normalize_hostname(hostname)
+        if hostname in self.managed_responses:
+            return self.managed_responses[hostname]
+        response_class = getattr(self,'managed_reponse_class',self.response_class)
+        response = response_class(default_level=self.managed_default_level)
+        self.managed_responses[hostname] = response
+        return response
+
+    def build_response(self,data):
+        self.init_managed_hosts(data)
+        self.build_manager_response(data)
+        self.build_managed_responses(data)
+
+    def get_plugin_managed_informations(self,response):
+        """Get plugin informations for managed hosts"""
+        managers = getattr(response,'managers',[self.host.name])
+        out = '\n' + response.section_format('Plugin Informations') + '\n'
+        out += 'This host is managed by : %s\n' % ','.join(managers)
+        out += 'Manager plugin name : %s.%s\n' % (self.__class__.__module__,self.__class__.__name__)
+        out += 'Update date : %s\n' % datetime.datetime.now()
+        level = response.get_current_level()
+        out += 'Response level : %s (%s), __sublevel__=%s' % (level.exit_code,level.name,response.sublevel)
+        return out
+
+    def send_managed_responses(self):
+        for hostname, response in self.managed_responses.items():
+            managed_host = self.managed_data.hosts[hostname]
+            self.debug('[%s] before : hash=%s,state=%s now : hash=%s,state=%s',hostname,managed_host.prev_hash,self.managed_nagios_states.get(hostname,0),managed_host.new_hash,managed_host.new_state)
+            if managed_host.new_hash != managed_host.prev_hash or self.managed_nagios_states.get(hostname,0) != managed_host.new_state:
+                self.debug('[%s] ---> sending alert',hostname)
+                response.add_end(self.get_plugin_managed_informations(response))
+                response.send(nagios_host=hostname,nagios_svc=self.managed_service_description,nagios_cmd=self.pynag_cmd)
+
+    def save_host_data(self):
+        self.save_managed_data()
+        super(HostsManagerMixin,self).save_host_data()
+
+    def send_response(self):
+        self.send_managed_responses()
+        super(HostsManagerMixin,self).send_response()
\ No newline at end of file
