@@ -26,6 +26,8 @@ import socket
 #
 pp = pprint.PrettyPrinter(indent=4)
 
+COLLECT_ALL_TIMEOUT = 900
+
 __all__ = [ 'ActivePlugin' ]
 
 class Plugin(object):
@@ -216,10 +218,10 @@ class Plugin(object):
                                             'name'  : member.__name__,
                                             'module': cls.plugins_basemodule + module_name,
                                             'path'  : os.sep.join(module_name.split('.'))+'.py',
-                                            'desc'  : doc.splitlines()[0]
+                                            'desc'  : doc.splitlines()[0] if doc else 'No description'
                                         }
                                 except Exception,e:
-                                    #print e
+                                    #traceback.print_exc()
                                     pass
                         except Exception,e:
                             #print e
@@ -257,7 +259,7 @@ class Plugin(object):
     @classmethod
     def get_plugin_desc(cls):
         """Returns the plugin description. By default return the class docstring. """
-        return cls.__doc__.strip() or ''
+        return cls.__doc__.strip() if cls.__doc__ else ''
 
     def init_cmd_options(self):
         """Create OptionParser instance and add some basic options
@@ -578,30 +580,32 @@ class ActivePlugin(Plugin):
     It is highly recommended to use the following parameters name as their description has been
     already defined by the method :meth:`get_plugin_host_params_tab` :
 
-    ==================  ==================================================
-     NAME                DESCRIPTION
-    ==================  ==================================================
-     name               Hostname
-     ip                 Host IP address
-     subtype            Plugin subtype (usually host model)
-     user               User
-     passwd             Password
-     console_ip         Console or controller IP address
-     snmpversion        SNMP protocal version (1,2 or 3)
-     community          SNMP community
-     community_alt      SNMP community for other device
-     authpp             SNMP authentification passphrase
-     authproto          SNMP authentification protocal (md5 or sha)
-     privpp             SNMP privacy passphrase
-     privproto          SNMP privacy protocal (des or aes)
-     protocol           ssh or telnet
-     port               Port number
-     maxwarn            Gauge max value for a warning status
-     maxcrit            Gauge max value for a critical status
-     minwarn            Gauge min value for a warning status
-     mincrit            Gauge min value for a critical status
-     options            Additionnal options
-    ==================  ==================================================
+    ====================  ==================================================
+     NAME                  DESCRIPTION
+    ====================  ==================================================
+     name                 Hostname
+     ip                   Host IP address
+     subtype              Plugin subtype (usually host model)
+     user                 User
+     passwd               Password
+     console_ip           Console or controller IP address
+     snmpversion          SNMP protocal version (1,2 or 3)
+     community            SNMP community
+     community_alt        SNMP community for other device
+     authpp               SNMP authentification passphrase
+     authproto            SNMP authentification protocal (md5 or sha)
+     privpp               SNMP privacy passphrase
+     privproto            SNMP privacy protocal (des or aes)
+     protocol             ssh or telnet
+     port                 Port number
+     collect_cmd_timeout  Maximum time allowed for one collect command
+     collect_all_timeout  Maximum time allowed for the whole collect process
+     maxwarn              Gauge max value for a warning status
+     maxcrit              Gauge max value for a critical status
+     minwarn              Gauge min value for a warning status
+     mincrit              Gauge min value for a critical status
+     options              Additionnal options
+    ====================  ==================================================
 
     Note that ``name`` and ``ip`` are hard coded :
     you must use them for Nagios hostname and hostaddress.
@@ -636,7 +640,7 @@ class ActivePlugin(Plugin):
     :attr:`cmd_params` are required.
     """
 
-    forced_params = 'name,ip'
+    forced_params = 'name,ip,collect_cmd_timeout,collect_all_timeout'
     """Attribute you can set to force all your plugins to have some default :class:`~naghelp.Host`
     parameters.
     These parameters are automatically added to the plugin attribute :attr:`cmd_params`.
@@ -706,26 +710,28 @@ class ActivePlugin(Plugin):
         If you want to create specific parameters, add them in the dictionary with their description
         by overriding this method in a subclass.
         """
-        return  {   'name'           : 'Hostname',
-                    'ip'             : 'Host IP address',
-                    'subtype'        : 'Plugin subtype (usually host model)',
-                    'user'           : 'User',
-                    'passwd'         : 'Password',
-                    'console_ip'     : 'Console or controller IP address',
-                    'snmpversion'    : 'SNMP protocal version (1,2 or 3)',
-                    'community'      : 'SNMP community',
-                    'community_alt'  : 'SNMP community for other device',
-                    'authpp'         : 'SNMP authentification passphrase',
-                    'authproto'      : 'SNMP authentification protocal (md5 or sha)',
-                    'privpp'         : 'SNMP privacy passphrase',
-                    'privproto'      : 'SNMP privacy protocal (des or aes)',
-                    'protocol'       : 'ssh or telnet',
-                    'port'           : 'Port number',
-                    'maxwarn'        : 'Gauge max value for a warning status',
-                    'maxcrit'        : 'Gauge max value for a critical status',
-                    'minwarn'        : 'Gauge min value for a warning status',
-                    'mincrit'        : 'Gauge min value for a critical status',
-                    'options'        : 'Additionnal options',
+        return  {   'name'                  : 'Hostname',
+                    'ip'                    : 'Host IP address',
+                    'subtype'               : 'Plugin subtype (usually host model)',
+                    'user'                  : 'User',
+                    'passwd'                : 'Password',
+                    'console_ip'            : 'Console or controller IP address',
+                    'snmpversion'           : 'SNMP protocal version (1,2 or 3)',
+                    'community'             : 'SNMP community',
+                    'community_alt'         : 'SNMP community for other device',
+                    'authpp'                : 'SNMP authentification passphrase',
+                    'authproto'             : 'SNMP authentification protocal (md5 or sha)',
+                    'privpp'                : 'SNMP privacy passphrase',
+                    'privproto'             : 'SNMP privacy protocal (des or aes)',
+                    'protocol'              : 'ssh or telnet',
+                    'port'                  : 'Port number',
+                    'collect_cmd_timeout'   : 'Maximum time allowed for one collect command',
+                    'collect_all_timeout'   : 'Maximum time allowed for the whole collect process',
+                    'maxwarn'               : 'Gauge max value for a warning status',
+                    'maxcrit'               : 'Gauge max value for a critical status',
+                    'minwarn'               : 'Gauge min value for a warning status',
+                    'mincrit'               : 'Gauge min value for a critical status',
+                    'options'               : 'Additionnal options',
                 }
 
     def get_plugin_host_params_desc(self):
@@ -745,8 +751,8 @@ class ActivePlugin(Plugin):
         This create :class:`optparse.OptionParser` instance and add some basic options
 
         It also add options corresponding to Host parameters. The host parameters will be stored
-        first into OptionParse's options object (``plugin.options``) at ``host__<parameter>`` attribute, later it is
-        set to host object at attribute ``<parameter>``
+        first into OptionParse's options object (``plugin.options``), later it is
+        set to host object.
 
         This method is automatically called when the plugin is run.
         Avoid to override this method, prefer to customize :meth:`add_cmd_options`
@@ -756,8 +762,8 @@ class ActivePlugin(Plugin):
         host_params_desc = self.get_plugin_host_params_desc()
         if host_params_desc:
             group = OptionGroup(self._cmd_parser, 'Host attributes','To be used to force host attributes values')
-            for param,desc in host_params_desc.items():
-                group.add_option('--%s' % param, action='store', type='string', dest="host__%s" % param, metavar=param.upper(), help=desc)
+            for param,desc in sorted(host_params_desc.items()):
+                group.add_option('--%s' % param, action='store', type='string', dest=param, metavar=param.upper(), help=desc)
             self._cmd_parser.add_option_group(group)
 
         self._cmd_parser.add_option('-n', action='store_true', dest='in_nagios_env',
@@ -843,7 +849,7 @@ class ActivePlugin(Plugin):
         msg_lines = msg.splitlines()
         synopsis = ''.join(msg_lines[:1])
         body = '\n'.join(msg_lines[1:])
-        if exception is None or not isinstance(exception, naghelp.CollectError):
+        if exception is None or not isinstance(exception, (naghelp.CollectError,naghelp.TimeoutError)):
             import traceback
             body += 'traceback : ' + traceback.format_exc() + '\n'
             if self.data:
@@ -1029,6 +1035,7 @@ class ActivePlugin(Plugin):
         out += 'Plugin name : %s.%s\n' % (self.__class__.__module__,self.__class__.__name__)
         out += 'Description : %s\n' % ( self.__class__.__doc__ or 'no description.' ).splitlines()[0].strip()
         out += 'Ports used : tcp = %s, udp = %s\n' % (self.get_tcp_ports() or 'none',self.get_udp_ports() or 'none')
+        out += 'Execution date : %s\n' % datetime.datetime.now()
         delta = datetime.datetime.now() - self.starttime
         out += 'Execution time : %s\n' % delta
         level = self.response.get_current_level()
@@ -1065,6 +1072,13 @@ class ActivePlugin(Plugin):
         """For doctest usage only"""
         self.host.save_data()
 
+    def save_host_data(self):
+        """Save data before sending the built response"""
+        self.host.save_data()
+
+    def send_response(self):
+        self.response.send()
+
     def run(self):
         """Run the plugin
 
@@ -1101,8 +1115,11 @@ class ActivePlugin(Plugin):
                 self.info('Collected data are restored')
             else:
                 try:
-                    self.collect_data(self.data)
+                    collect_timeout = int(self.host.collect_all_timeout or COLLECT_ALL_TIMEOUT)
+                    with naghelp.Timeout(seconds=collect_timeout, error_message='Collect process timeout'):
+                        self.collect_data(self.data)
                 except Exception,e:
+                    self.debug('Collect exception : %s',e)
                     if self.get_tcp_ports():
                         self.info('Checking TCP ports %s ...' % self.get_tcp_ports())
                         self.check_ports()
@@ -1136,9 +1153,9 @@ class ActivePlugin(Plugin):
                 exit(0)
 
             self.build_response(self.data)
-            self.host.save_data()
+            self.save_host_data()
             self.response.add_end(self.get_plugin_informations())
-            self.response.send()
+            self.send_response()
         except Exception,e:
             self.error('Plugin internal error : %s' % e, exception=e)
 
