@@ -5,13 +5,18 @@
 # @author: Eric Lapouyade
 #
 
-import sys
+import sys,os
 import naghelp
 from types import NoneType
 import re
+import traceback
+import hashlib
+from datetime import datetime
 
 __all__ = [ 'ResponseLevel', 'PluginResponse', 'OK', 'WARNING', 'CRITICAL', 'UNKNOWN', 'LevelComment' ]
 
+MAX_PIPE_OUTPUT_LENGTH = 7000
+
 class ResponseLevel(object):
     """Object to use when exiting a naghelp plugin
 
@@ -187,14 +192,14 @@ class PluginResponse(object):
 
         Actually, the sublevel meanings are :
 
-        =========  ===========================================================================
+        =========  ====================================================================================================
         Sub-level  Description
-        =========  ===========================================================================
+        =========  ====================================================================================================
         0          The plugin is 100% sure there is a critical error
-        1          The plugin was able to contact remote host but got no answer from agent
-        2          The plugin was unable to contact the remote host, it may be a network issue
+        1          The plugin was able to contact remote host (ping) but got no answer from agent (timeout,credentials)
+        2          The plugin was unable to contact the remote host, it may be a network issue (firewall, ...)
         3          The plugin raised an unexpected exception : it should be a bug.
-        =========  ===========================================================================
+        =========  ====================================================================================================
         """
         if not isinstance(sublevel,int):
             raise Exception('A response sublevel must be an integer')
@@ -268,6 +273,7 @@ class PluginResponse(object):
             <BLANKLINE>
             <BLANKLINE>
         """
+        naghelp.logger.debug('response -> add_begin("%s") %s',msg,naghelp.debug_caller())
         self.begin_msgs.append(self._reformat_msg(msg,*args,**kwargs))
 
     def add(self,level,msg,*args,**kwargs):
@@ -317,6 +323,8 @@ class PluginResponse(object):
             <BLANKLINE>
             <BLANKLINE>
         """
+        if not kwargs.get('no_debug'):
+            naghelp.logger.debug('response -> add(%s,"%s") %s',level,msg,naghelp.debug_caller())
         if isinstance(level,ResponseLevel):
             self.level_msgs[level].append(self._reformat_msg(msg,*args,**kwargs))
             self.set_level(level)
@@ -356,6 +364,7 @@ class PluginResponse(object):
             <BLANKLINE>
             <BLANKLINE>
         """
+        naghelp.logger.debug('response -> add_comment(%s,"%s") %s',level,msg,naghelp.debug_caller())
         if isinstance(level,ResponseLevel):
             self.level_msgs[level].append(LevelComment(self._reformat_msg(msg,*args,**kwargs)))
         else:
@@ -441,15 +450,94 @@ class PluginResponse(object):
 
         have_added=False
         kwargs['_len'] = len(msg_list)
-        for msg in msg_list:
+        naghelp.logger.debug('response -> add_list(...) %s',naghelp.debug_caller())
+        for i,msg in enumerate(msg_list):
+            naghelp.logger.debug('response -> [%s] %s',i,msg)
             if msg:
                 if not have_added and header is not None:
                     self.add_comment(level, header,*args,**kwargs)
-                self.add(level, msg,*args,**kwargs)
+                self.add(level, msg,no_debug=True,*args,**kwargs)
                 have_added = True
         if have_added and footer is not None:
             self.add_comment(level, footer,*args,**kwargs)
 
+    def add_mlist(self,levels,lists,headers=[],footers=[],*args,**kwargs):
+        """Add a list of lists of messages
+
+        This is useful with :class:`textops.sgrep` utility that returns lists of lists of messages.
+        first level will be affected to first list, second level to the second list and so on.
+
+        Args:
+
+            levels (ResponseLevel): a list of message levels. use a ``None`` level to skip a list.
+            lists (list): list of message lists.
+            header (str): Displayed before the message as a level comment if not None (Default : None)
+              one can use ``{_len}`` in the comment to get list count.
+            footer (str): Displayed after the message as a level comment if not None (Default : None)
+              one can use ``{_len}`` in the comment to get list count.
+            args (list): if additional arguments are given, messages in ``msg_list``
+                will be formatted with ``%`` (old-style python string formatting)
+            kwargs (dict): if named arguments are given, messages in ``msg_list``
+                will be formatted with :meth:`str.format`
+
+
+        Examples:
+
+            >>> from textops import StrExt
+            >>> r = PluginResponse(OK)
+            >>> logs = StrExt('''
+            ... Power 0 is critical
+            ... Power 1 is OK
+            ... Power 2 is degraded
+            ... Power 3 is failed
+            ... Power 4 is OK
+            ... Power 5 is degraded
+            ... ''')
+            >>>
+            >>> print logs.sgrep(('critical|failed','degraded|warning'))  #doctest: +NORMALIZE_WHITESPACE
+            [['Power 0 is critical', 'Power 3 is failed'],
+            ['Power 2 is degraded', 'Power 5 is degraded'],
+            ['', 'Power 1 is OK', 'Power 4 is OK']]
+            >>> r.add_mlist((CRITICAL,WARNING,OK),logs.sgrep(('critical|failed','degraded|warning')))
+            >>> print r                                                   #doctest: +NORMALIZE_WHITESPACE
+            STATUS : CRITICAL:2, WARNING:2, OK:2
+            ==================================[  STATUS  ]==================================
+            <BLANKLINE>
+            ----( CRITICAL )----------------------------------------------------------------
+            Power 0 is critical
+            Power 3 is failed
+            <BLANKLINE>
+            ----( WARNING )-----------------------------------------------------------------
+            Power 2 is degraded
+            Power 5 is degraded
+            <BLANKLINE>
+            ----( OK )----------------------------------------------------------------------
+            Power 1 is OK
+            Power 4 is OK
+            <BLANKLINE>
+            >>> r = PluginResponse(OK)
+            >>> r.add_mlist((CRITICAL,WARNING,None),logs.sgrep(('critical|failed','degraded|warning')))
+            >>> print r
+            STATUS : CRITICAL:2, WARNING:2
+            ==================================[  STATUS  ]==================================
+            <BLANKLINE>
+            ----( CRITICAL )----------------------------------------------------------------
+            Power 0 is critical
+            Power 3 is failed
+            <BLANKLINE>
+            ----( WARNING )-----------------------------------------------------------------
+            Power 2 is degraded
+            Power 5 is degraded
+            <BLANKLINE>
+            <BLANKLINE>
+        """
+        for i,level in enumerate(levels):
+            if level is not None:
+                msg_list = lists[i] if i<len(lists) else []
+                header = headers[i] if i<len(headers) else None
+                footer = footers[i] if i<len(footers) else None
+                self.add_list(level,msg_list,header,footer,*args,**kwargs)
+
     def add_many(self,lst,*args,**kwargs):
         """Add several level messages NOT having a same level
 
@@ -501,7 +589,9 @@ class PluginResponse(object):
             <BLANKLINE>
             <BLANKLINE>
         """
-        for level,msg in lst:
+        naghelp.logger.debug('response -> add_many(...) %s',naghelp.debug_caller())
+        for i,(level,msg) in enumerate(lst):
+            naghelp.logger.debug('response -> [%s] (%s,"%s")',i,level,msg)
             if msg:
                 self.add(level, msg,*args,**kwargs)
 
@@ -566,6 +656,7 @@ class PluginResponse(object):
             <BLANKLINE>
             <BLANKLINE>
         """
+        naghelp.logger.debug('response -> add_if(%s,%s,"%s") %s',test,level,msg,naghelp.debug_caller())
         if msg is None:
             msg = test
         if isinstance(level,ResponseLevel):
@@ -636,7 +727,9 @@ class PluginResponse(object):
             <BLANKLINE>
             <BLANKLINE>
         """
-        for test,level,msg in add_ifs:
+        naghelp.logger.debug('response -> add_elif(...) %s',naghelp.debug_caller())
+        for i,(test,level,msg) in enumerate(add_ifs):
+            naghelp.logger.debug('response -> [%s] (%s,%s,"%s")',i,test,level,msg)
             if msg is None:
                 msg = test
             if isinstance(level,ResponseLevel):
@@ -682,6 +775,9 @@ class PluginResponse(object):
             ==========================[ Additionnal informations ]==========================
             Date : 2105-12-18, Time : 14:55:11
         """
+        if not kwargs.get('no_debug'):
+            naghelp.logger.debug('response -> add_more("%s") %s',msg,naghelp.debug_caller())
+
         if msg:
             if isinstance(msg,(list,tuple)):
                 msg = '\n'.join(msg)
@@ -731,6 +827,7 @@ class PluginResponse(object):
             ========================================
             Date : 2105-12-18, Time : 14:55:11
         """
+        naghelp.logger.debug('response -> add_more("%s") %s',msg,naghelp.debug_caller())
         if isinstance(msg,(list,tuple)):
             msg = '\n'.join(msg)
         elif not isinstance(msg,basestring):
@@ -767,6 +864,7 @@ class PluginResponse(object):
             cpu_wait=88%;40;60;0;100
             cpu_user=12%;80;95;0;100
         """
+        naghelp.logger.debug('response -> add_perf_data("%s") %s',data,naghelp.debug_caller())
         if not isinstance(data,basestring):
             data = str(data)
         self.perf_items.append(data)
@@ -956,7 +1054,7 @@ class PluginResponse(object):
         """
         return msg.replace('|','!')
 
-    def get_output(self):
+    def get_output(self,body_max_length=None):
         r"""Renders the whole response following the Nagios syntax
 
         This method is automatically called when the response is sent by :meth:`send`.
@@ -1012,7 +1110,7 @@ class PluginResponse(object):
             synopsis_start += '...'
 
         out = self.escape_msg(synopsis_start)
-        out +=  '|%s\n' % self.perf_items[0] if self.perf_items else '\n'
+        out += '|%s\n' % self.perf_items[0] if self.perf_items else '\n'
 
         if synopsis_first_line[synopsis_maxlen:]:
             out += '... %s\n' % self.escape_msg(synopsis_first_line[synopsis_maxlen:])
@@ -1022,16 +1120,23 @@ class PluginResponse(object):
         if self.more_msgs:
             body += self.section_format('Additionnal informations') + '\n'
             body += '\n'.join(self.more_msgs)
+
+        if body_max_length and len(body) > body_max_length:
+            body = body[:body_max_length] + '...\n\n--- Message is too big for nagios : it has been truncated ---\n\n'
+
         body += '\n'.join(self.end_msgs)
 
         out += self.escape_msg(body)
-        out +=  '|%s' % '\n'.join(self.perf_items[1:]) if len(self.perf_items)>1 else ''
+        out += '|%s' % ' ' + ' '.join(self.perf_items[1:]) if len(self.perf_items)>1 else ''
         return out
 
     def __str__(self):
         return self.get_output()
 
-    def send(self, level=None, synopsis='', msg='', sublevel = None):
+    def get_hash(self):
+        return hashlib.md5(self.get_output()).hexdigest()
+
+    def send(self, level=None, synopsis='', msg='', sublevel=None, nagios_host=None, nagios_svc=None, nagios_cmd=None):
         r"""Send the response to Nagios
 
         This method is automatically called by :meth:`naghelp.ActivePlugin.run` method and
@@ -1048,6 +1153,12 @@ class PluginResponse(object):
             synopsis(str): force a synopsis (optional),
             msg(str): add a last level message (optional),
             sublevel(int): force a sublevel [0-3] (optional),
+            nagios_host (str): nagios hostname (only for passive response)
+            nagios_svc (str): nagios service (only for passive response)
+            nagios_cmd (str or pynag obj): if None, the response goes to stdout : this is for
+                active plugin. if string the response is sent to the corresponding file for debug.
+                if it is a pynag cmd object, the response is sent to the nagios pipe for
+                host ``nagios_host`` and for service ``nagios_svc`` : this is for a passive plugin.
         """
         if isinstance(level,ResponseLevel):
             self.set_level(level)
@@ -1062,11 +1173,24 @@ class PluginResponse(object):
 
         naghelp.logger.info('Plugin output summary : %s' % self.synopsis)
 
-        out = self.get_output()
+        out = self.get_output(MAX_PIPE_OUTPUT_LENGTH if nagios_cmd else None)
 
         naghelp.logger.debug('Plugin output :\n' + '#' * 80 + '\n' + out + '\n'+ '#' * 80)
 
-        print out.encode('utf-8') if isinstance(out,unicode) else out
-
-        naghelp.logger.info('Exiting plugin with response level : %s, __sublevel__=%s', self.level.info(), self.sublevel )
-        self.level.exit()
+        if nagios_cmd:
+            # if nagios_cmd is a string, response will be added into the specified file (useful for testing)
+            if isinstance(nagios_cmd,basestring):
+                naghelp.logger.debug('Sending response for %s/%s in file %s ...',nagios_host,nagios_svc,nagios_cmd)
+                with open(nagios_cmd,'w+') as fh:
+                    fh.write('='*80)
+                    fh.write('[%s] Response for %s/%s' % (datetime.now(),nagios_host,nagios_svc))
+                    fh.write('-'*80)
+                    fh.write(out)
+            else:
+                naghelp.logger.debug('Sending response to command pipe for %s/%s ...',nagios_host,nagios_svc)
+                nagios_cmd.process_service_check_result(nagios_host,nagios_svc,self.level.exit_code,out.replace('\n',r'\n'))
+        else:
+            naghelp.logger.debug('Sending response on stdout...')
+            print out.encode('utf-8') if isinstance(out,unicode) else out
+            naghelp.logger.info('Exiting plugin with response level : %s, __sublevel__=%s', self.level.info(), self.sublevel )
+            self.level.exit()
